@@ -515,7 +515,7 @@ nm_wifi_ap_to_string(const NMWifiAP *self, char *str_buf, gulong buf_len, gint64
 
     g_snprintf(str_buf,
                buf_len,
-               "%17s %-35s [ %c %3u %3u%% %c%c W:%04X R:%04X ] %s sup:%s [nm:%s]",
+               "%17s %-35s [ %c %3u %3u%% %c%c %c%c W:%04X R:%04X ] %s sup:%s [nm:%s]",
                priv->address ?: "(none)",
                (ssid_to_free = _nm_utils_ssid_to_string(priv->ssid)),
                (priv->mode == NM_802_11_MODE_ADHOC
@@ -527,6 +527,10 @@ nm_wifi_ap_to_string(const NMWifiAP *self, char *str_buf, gulong buf_len, gint64
                priv->strength,
                priv->flags & NM_802_11_AP_FLAGS_PRIVACY ? 'P' : '_',
                priv->metered ? 'M' : '_',
+               priv->flags & NM_802_11_AP_FLAGS_WPS ? 'W' : '_',
+               priv->flags & NM_802_11_AP_FLAGS_WPS_PIN
+                   ? 'p'
+                   : (priv->flags & NM_802_11_AP_FLAGS_WPS_PBC ? '#' : '_'),
                priv->wpa_flags & 0xFFFF,
                priv->rsn_flags & 0xFFFF,
                priv->last_seen_msec != G_MININT64
