@@ -11,7 +11,7 @@
 #include "nm-glib-aux/nm-enum-utils.h"
 #include "nm-core-internal.h"
 #include "platform/nm-platform.h"
-#include "platform/nm-platform-utils.h"
+#include "nm-platform/nm-platform-utils.h"
 #include "platform/nmp-object.h"
 #include "NetworkManagerUtils.h"
 
