@@ -9,9 +9,9 @@
 
 #include <linux/if.h>
 
+#include "nm-platform/nm-netlink.h"
 #include "platform/linux/nl802154.h"
-#include "platform/nm-netlink.h"
-#include "platform/nm-platform-utils.h"
+#include "nm-platform/nm-platform-utils.h"
 
 #define _NMLOG_PREFIX_NAME "wpan-nl802154"
 #define _NMLOG(level, domain, ...)                                                                \
